//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxMailTools
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2006-06-05 10:15:51 +0200 (Mo, 05 Jun 2006) $
 *   $Revision: 219 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <algorithm>
#include <string>

//----- C ---------------------------------------------------------------------
#include <iconv.h>
#include <errno.h>
#include <ctype.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include <Ax/Mail/Tools.h>
#include <Ax/Mail/Globals.h>

//----- local -----------------------------------------------------------------
#include "c-client-header.h"

//=============================================================================
//     using namespaces
//=============================================================================

//=============================================================================
//     namespaces
//=============================================================================
namespace Ax
{
namespace Mail
{
namespace Tools
{

//-----------------------------------------------------------------------
//     typedefs
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//     map2CharSet()
//-----------------------------------------------------------------------
bool map2CharSet( const std::string  &theSrc
                , const std::string  &theSrcCharSet
                ,       std::string  &theDst
                , const std::string  &theDstCharSet
                )
{
  wsdebug(("> Mail::Tools::map2CharSet() Src: '%s'", theSrc.c_str()));
  wsdebug(("  Mail::Tools::map2CharSet() SrcCharSet: '%s'", theSrcCharSet.c_str()));
  wsdebug(("  Mail::Tools::map2CharSet() SrcCharSet: '%s'", theDstCharSet.c_str()));

  // check some short-cuts
  if (theSrc.empty())
  {
    wsdebug(("< Mail::Tools::map2CharSet() empty input -> returning!"));
    theDst = theSrc;
    return true;
  } // if

  if (theSrcCharSet.empty())
  {
    wsdebug(("< Mail::Tools::map2CharSet() empty src-charset -> returning!"));
    theDst = theSrc;
    return false;
  } // if

  if (theDstCharSet.empty())
  {
    wsdebug(("< Mail::Tools::map2CharSet() empty dst-charset -> returning!"));
    theDst = theSrc;
    return false;
  } // if

  std::string aSrcCharSet = theSrcCharSet;
  std::string aDstCharSet = theDstCharSet;

  std::transform(aSrcCharSet.begin(), aSrcCharSet.end(), aSrcCharSet.begin(), toupper);
  std::transform(aDstCharSet.begin(), aDstCharSet.end(), aDstCharSet.begin(), toupper);

  if (aSrcCharSet == aDstCharSet)
  {
    wsdebug(("< Mail::Tools::map2CharSet() same charset -> returning input unchanged!"));
    theDst = theSrc;
    return true;
  } // if

  iconv_t aCD = iconv_open(aDstCharSet.c_str(), aSrcCharSet.c_str());
  if (aCD == (iconv_t)-1)
  {
    wserror(("ERROR: iconv conversion not supported from '%s' to '%s': %m"
           , aSrcCharSet.c_str()
           , aDstCharSet.c_str()
           ));
    return false;
  } // if

  bool        fOK     = true;
  std::string aDst    = "";

  size_t      aCount  = 0;

  size_t      aSizeIn = theSrc.length();
  char       *aBufIn  = (char *)malloc(aSizeIn + 1);
  char       *aPtrIn  = aBufIn;

  memcpy(aBufIn, theSrc.c_str(), aSizeIn);
  aBufIn[aSizeIn]     = '\0';

  const size_t BUF_SIZE = 1024;
  size_t      aSizeOut  = BUF_SIZE;
  char         aBufOut[BUF_SIZE + 1];
  char        *aPtrOut  = aBufOut;

  memset(aBufOut, 0, sizeof(aBufOut));

  while (true)
  {
    wsdebug(("InBuf   : '%s'", aPtrIn ));

    aCount = iconv(aCD, &aPtrIn , &aSizeIn
                  ,     &aPtrOut, &aSizeOut
                  );
    int anErrNo = errno;

    // evaluation of the results according to "man 3 iconv"

    // case 1: invalid multibyte sequence -> abort
    if (aCount == (size_t)-1 && anErrNo == EILSEQ)
    {
      wserror(("ERROR: iconv (case 1) returned: %m"));
      fOK = false;
      break;
    } // if

    // case 2: OK, conversion finished
    if (aCount != (size_t)-1 && aSizeIn == 0)
    {
      aDst += std::string(aBufOut);
      fOK = true;
      break;
    } // if

    // case 3: incomplete multibyte sequence -> abort
    if (aCount == (size_t)-1 && anErrNo == EINVAL)
    {
      wserror(("ERROR: iconv (case 3) returned: %m"));
      fOK = false;
      break;
    } // if

    // case 4: partially converted, there's more to do
    if (aCount == (size_t)-1 && anErrNo == E2BIG)
    {
      aDst += std::string(aBufOut);

      memset(aBufOut, 0, sizeof(aBufOut));
      aPtrOut  = aBufOut;
      aSizeOut = BUF_SIZE;

      continue;
    } // if

    // unknown state -> abort
    wserror(("ERROR: unknown state while using iconv"));
    fOK = false;
    break;
  } // while

  free(aBufIn);
  iconv_close(aCD);

  if (fOK)
  {
    theDst = aDst;
  }
  else
  {
    theDst = theSrc;
  } // if

  wsdebug(("  Mail::Tools::map2CharSet() Dst: '%s'", theDst.c_str()));
  wsdebug(("< Mail::Tools::map2CharSet() fOK: %d", fOK));
  return fOK;
} // map2CharSet()

//-----------------------------------------------------------------------
//     encodeBase64()
//-----------------------------------------------------------------------
bool encodeBase64(const std::string &theSrc, std::string &theDst)
{
  wsdebug(("> Mail::Tools::encodeBase64() Src: '%s'", theSrc.c_str()));
  bool        fOK     = true;
  std::string aDst    = "";

  size_t         aSizeIn = theSrc.length();
  unsigned char *aBufIn  = (unsigned char *)malloc(aSizeIn + 1);
  memcpy(aBufIn, theSrc.c_str(), aSizeIn);
  aBufIn[aSizeIn]     = '\0';

  unsigned long aDecLen = 0;
  char *aDecoded = (char *)rfc822_binary(aBufIn, aSizeIn, &aDecLen);

  if (aDecoded)
  {
    theDst = std::string(aDecoded, aDecLen);
    fs_give((void **)&aDecoded);
  }
  else
  {
    fOK    = false;
    theDst = theSrc;
    wswarn(("Mail::Tools::encodeBase64() decoding of 'Base64' failed"));
  } // if

  wsdebug(("  Mail::Tools::encodeBase64() Dst: '%s'", theDst.c_str()));
  wsdebug(("< Mail::Tools::encodeBase64() fOK: %d", fOK));
  return fOK;
} // encodeBase64()

//-----------------------------------------------------------------------
//     decodeBase64()
//-----------------------------------------------------------------------
bool decodeBase64(const std::string &theSrc, std::string &theDst)
{
  wsdebug(("> Mail::Tools::decodeBase64() Src: '%s'", theSrc.c_str()));
  bool        fOK     = true;
  std::string aDst    = "";

  size_t         aSizeIn = theSrc.length();
  unsigned char *aBufIn  = (unsigned char *)malloc(aSizeIn + 1);
  memcpy(aBufIn, theSrc.c_str(), aSizeIn);
  aBufIn[aSizeIn]     = '\0';

  unsigned long aDecLen = 0;
  char *aDecoded = (char *)rfc822_base64(aBufIn, aSizeIn, &aDecLen);

  if (aDecoded)
  {
    theDst = std::string(aDecoded);
    fs_give((void **)&aDecoded);
  }
  else
  {
    fOK    = false;
    theDst = theSrc;
    wswarn(("Mail::Tools::decodeBase64() decoding of 'Base64' failed"));
  } // if

  wsdebug(("  Mail::Tools::decodeBase64() Dst: '%s'", theDst.c_str()));
  wsdebug(("< Mail::Tools::decodeBase64() fOK: %d", fOK));
  return fOK;
} // decodeBase64()

//-----------------------------------------------------------------------
//     decodeQuoted()
//-----------------------------------------------------------------------
bool decodeQuoted(const std::string &theSrc, std::string &theDst)
{
  wsdebug(("> Mail::Tools::decodeQuoted() Src: '%s'", theSrc.c_str()));

  bool fOK = true;

  size_t         aSizeIn = theSrc.length();
  unsigned char *aBufIn  = (unsigned char *)malloc(aSizeIn + 1);
  memcpy(aBufIn, theSrc.c_str(), aSizeIn);
  aBufIn[aSizeIn]     = '\0';

  unsigned long aDecLen = 0;
  char *aDecoded = (char *)rfc822_qprint(aBufIn, aSizeIn, &aDecLen);

  if (aDecoded)
  {
    theDst = std::string(aDecoded);
    fs_give((void **)&aDecoded);
  }
  else
  {
    fOK    = false;
    theDst = theSrc;
    wswarn(("Mail::Tools::decodeQuoted() decoding of 'quoted-printable' failed"));
  } // if

  wsdebug(("  Mail::Tools::decodeQuoted() Dst: '%s'", theDst.c_str()));
  wsdebug(("< Mail::Tools::decodeQuoted() fOK: %d", fOK));
  return fOK;
} // decodeQuoted()

//-----------------------------------------------------------------------
//     decodeEncodedWord()
//-----------------------------------------------------------------------
bool decodeEncodedWord( const std::string  &theSrc
                      , const std::string  &theEncoding
                      ,       std::string  &theDst
                      )
{
  wsdebug(("> Mail::Tools::decodeEncodedWord() Src: '%s'", theSrc.c_str()));
  wsdebug(("  Mail::Tools::decodeEncodedWord() Env: '%s'", theEncoding.c_str()));

  bool fOK = false;
  if (theEncoding == "Q")
  {
    // Q = quoted encoding
    fOK = Ax::Mail::Tools::decodeQuoted(theSrc, theDst);
  }
  else if (theEncoding == "B")
  {
    // B = Base64 encoding
    fOK = decodeBase64(theSrc, theDst);
  }
  else
  {
    // invalid encoding
    wswarn(("Mail::Tools::decodeEncodedWord() invalid encoding '%s'", theEncoding.c_str()));
    fOK    = false;
    theDst = theSrc;
  } // if

  wsdebug(("  Mail::Tools::decodeEncodedWord() Dst: '%s'", theDst.c_str()));
  wsdebug(("< Mail::Tools::decodeEncodedWord() fOK: %d", fOK));
  return fOK;
} // AxMailTools::decodeEncodedWord()

//-----------------------------------------------------------------------
//     decodeEncodedWords()
//-----------------------------------------------------------------------
bool decodeEncodedWords( const std::string  &theSrc
                       ,       std::string  &theDst
                       , const std::string  &theDstCharSet
                       , bool                fMsgHeader
                       )
{
  wsdebug(("> Mail::Tools::decodeEncodedWords() Src: '%s'", theSrc.c_str()));
  bool fOK      = true;
  bool fHasCode = false;
  std::string::size_type aPosStart = 0;

  std::string aDst = theSrc;

  // in message header we don't expect tab-characters
  // these may occur, if e.g. the subject line is wrapped
  // into multiple lines.
  if (fMsgHeader)
  {
    std::string::iterator aPos = std::remove(aDst.begin(), aDst.end(), '\t');
    if (aPos != aDst.end()) aDst.erase(aPos, aDst.end());
  } // if

  while ((aPosStart = aDst.find("=?", aPosStart)) != std::string::npos)
  {
    std::string::size_type aStrLen = aDst.length();
    std::string::size_type aPosQ1  = aPosStart == std::string::npos || aPosStart + 2 > aStrLen ? std::string::npos : aDst.find("?" , aPosStart + 2);
    std::string::size_type aPosQ2  = aPosQ1    == std::string::npos || aPosQ1    + 1 > aStrLen ? std::string::npos : aDst.find("?" , aPosQ1    + 1);
    std::string::size_type aPosEnd = aPosQ2    == std::string::npos || aPosQ2    + 1 > aStrLen ? std::string::npos : aDst.find("?=", aPosQ2    + 1);

    wsdebug(("  Mail::Tools::decodeEncodedWords() Start: %3d  Q1: %3d  Q2: %3d  End: %3d  Len: %d"
          , aPosStart, aPosQ1, aPosQ2, aPosEnd, aStrLen
          ));
    wsdebug(("  Mail::Tools::decodeEncodedWords() Remaining: '%s'", &(aDst.c_str()[aPosStart])));

    if ( aPosStart != std::string::npos
      && aPosQ1    != std::string::npos
      && aPosQ2    != std::string::npos
      && aPosEnd   != std::string::npos
       )
    {
      wsdebug(("  Mail::Tools::decodeEncodedWords() Start: %3d  Q1: %3d  Q2: %3d  End: %3d  Len: %d"
             , aPosStart, aPosQ1, aPosQ2, aPosEnd, aStrLen
             ));

      std::string aCharSet  = aDst.substr(aPosStart + 2, aPosQ1  - aPosStart - 2);
      std::string aEncoding = aDst.substr(aPosQ1    + 1, aPosQ2  - aPosQ1    - 1);
      std::string aText     = aDst.substr(aPosQ2    + 1, aPosEnd - aPosQ2    - 1);

      // convert charset and encoding to upper case
      std::transform(aCharSet .begin(), aCharSet .end(), aCharSet .begin(), toupper);
      std::transform(aEncoding.begin(), aEncoding.end(), aEncoding.begin(), toupper);

      wsdebug(("  Mail::Tools::decodeEncodedWords() charset: '%s'  encoding: '%s'  Text: '%s'"
             , aCharSet.c_str(), aEncoding.c_str(), aText.c_str()
             ));

      fHasCode = true;

      // in message titles spaces are encoded as '_' whereas '_' is encoded in '=5F'
      // so it's necessary to replace all '_' with ' '
      if (fMsgHeader)
      {
        std::replace(aText.begin(), aText.end(), '_', ' ');
      } // if

      if (!decodeEncodedWord(aText, aEncoding, aText))
      {
        wswarn(("Mail::Tools::decodeEncodedWords() unable to convert encoded words: charset: %-14s  encoding: '%s'  Text: '%s'"
              , aCharSet.c_str(), aEncoding.c_str(), aText.c_str()
              ));
        // continue after the current encoded word
        aPosStart = aPosEnd;
        fOK = false;
      }
      else
      {
        wsdebug(("  Mail::Tools::decodeEncodedWords() converting charset..."));
        if (!map2CharSet(aText, aCharSet, aText, theDstCharSet))
        {
          wswarn(("Mail::Tools::decodeEncodedWords() unable to convert charset: %-14s  encoding: '%s'  Text: '%s'"
                , aCharSet.c_str(), aEncoding.c_str(), aText.c_str()
                ));
        } // if

        wsdebug(("  Mail::Tools::decodeEncodedWords() replacing with: '%s'"
               , aText.c_str()
               ));

        // replace the current encoded word
        aDst.replace(aPosStart, aPosEnd - aPosStart + 2, aText);

        aPosStart += aText.length();
      } // if
    }
    else if (aPosEnd == std::string::npos)
    {
      // no further tries needed
      break;
    } // if
  } // while

  if (fOK)
  {
    theDst = aDst;
  }
  else
  {
    theDst = theSrc;
  } // if

  wsdebug(("  Mail::Tools::decodeEncodedWords() Dst: '%s'", theDst.c_str()));
  wsdebug(("< Mail::Tools::decodeEncodedWords() fOK: %d", fOK));
  return fOK;
} // AxMailTools::decodeEncodedWords()


}; // namespace Tools
}; // namespace Mail
}; // namespace Ax
